@@ -19,6 +19,7 @@ import threading
 from collections import defaultdict
 from functools import wraps
 
+from cloudify.utils import get_func
 from cloudify.exceptions import WorkflowFailed
 from cloudify.workflows import api
 from cloudify.workflows import tasks
@@ -77,6 +78,33 @@ class TaskDependencyGraph(object):
         self._waiting_for = set()
         self._tasks_wait = threading.Event()
         self._finished_tasks = {}
+        self._op_types_cache = {}
+
+    def linearize(self):
+        """Traverse the graph, and return tasks in dependency order.
+
+        This makes sure that if task A depends on task B, then A is
+        going to be after B in the resulting list. Ordering of tasks
+        that are not related by dependencies is undefined.
+
+        This is useful for logging, debugging, and testing.
+        """
+        dependencies_copy = {k: set(v) for k, v in self._dependencies.items()}
+        ordered = []
+        ready = [
+            task for task in self._tasks.values()
+            if not dependencies_copy.get(task)
+        ]
+        while ready:
+            new_ready = []
+            for task in ready:
+                ordered.append(task)
+                for dependent_task in self._dependents[task]:
+                    dependencies_copy[dependent_task].discard(task)
+                    if not dependencies_copy.get(dependent_task):
+                        new_ready.append(dependent_task)
+            ready = new_ready
+        return ordered
 
     def _restore_dependencies(self, ops):
         """Set dependencies between this graph's tasks according to ops.
@@ -155,9 +183,26 @@ class TaskDependencyGraph(object):
         restored = self.get_task(op_descr.id)
         if restored is not None:
             return restored
-        return OP_TYPES[op_descr.type].restore(
+        op_cls = self._get_operation_class(op_descr.type)
+        return op_cls.restore(
             self.ctx._get_current_object(), self, op_descr)
 
+    def _get_operation_class(self, task_type):
+        if task_type in self._op_types_cache:
+            return self._op_types_cache[task_type]
+        if task_type == 'SubgraphTask':
+            op_cls = SubgraphTask
+        elif '.' in task_type:
+            op_cls = get_func(task_type)
+        else:
+            op_cls = getattr(tasks, task_type)
+
+        if not issubclass(op_cls, tasks.WorkflowTask):
+            raise RuntimeError('{0} is not a subclass of WorkflowTask'
+                               .format(task_type))
+        self._op_types_cache[task_type] = op_cls
+        return op_cls
+
     def store(self, name):
         serialized_tasks = []
         for task in self._tasks.values():
@@ -220,6 +265,8 @@ class TaskDependencyGraph(object):
         :param src_task: The source task
         :param dst_task: The target task
         """
+        if src_task == dst_task:
+            return
         if src_task.id not in self._tasks:
             raise RuntimeError('src not in graph: {0!r}'.format(src_task))
         if dst_task.id not in self._tasks:
@@ -507,11 +554,3 @@ class SubgraphTask(tasks.WorkflowTask):
 
 def _on_failure_handler_fail(task):
     return tasks.HandlerResult.fail()
-
-
-OP_TYPES = {
-    'RemoteWorkflowTask': tasks.RemoteWorkflowTask,
-    'LocalWorkflowTask': tasks.LocalWorkflowTask,
-    'NOPLocalWorkflowTask': tasks.NOPLocalWorkflowTask,
-    'SubgraphTask': SubgraphTask
-}
