@@ -146,7 +146,8 @@ class NodeInstancesClient(object):
                node_instance_id,
                state=None,
                runtime_properties=None,
-               version=1):
+               version=1,
+               force=False):
         """
         Update node instance with the provided state & runtime_properties.
 
@@ -155,6 +156,7 @@ class NodeInstancesClient(object):
         :param runtime_properties: The updated runtime properties.
         :param version: Current version value of this node instance in
          Cloudify's storage (used for optimistic locking).
+        :param force: ignore the version check - use with caution
         :return: The updated node instance.
         """
         assert node_instance_id
@@ -164,7 +166,10 @@ class NodeInstancesClient(object):
             data['runtime_properties'] = runtime_properties
         if state is not None:
             data['state'] = state
-        response = self.api.patch(uri, data=data)
+        params = {}
+        if force:
+            params['force'] = True
+        response = self.api.patch(uri, params=params, data=data)
         return NodeInstance(response)
 
     def _create_filters(
@@ -223,3 +228,27 @@ class NodeInstancesClient(object):
             [self._wrapper_cls(item) for item in response['items']],
             response['metadata']
         )
+
+    def search(self, ids, all_tenants=False):
+        """Search node instances by their IDs.
+
+        :param ids: list of ids to search by
+        :param all_tenants: search node-instances of all tenants
+        :return: Node instances.
+        :rtype: list
+        """
+        params = {}
+        if all_tenants:
+            params['_all_tenants'] = True
+        response = self.api.post('/searches/node-instances', data={
+            'filter_rules': [{
+                'key': 'id',
+                'values': ids,
+                'operator': 'any_of',
+                'type': 'attribute'
+            }]
+        }, params=params)
+        return ListResponse(
+            [self._wrapper_cls(item) for item in response['items']],
+            response['metadata']
+        )
